@@ -7,6 +7,8 @@ import tarfile
 import pyfastaq
 import time
 import json
+import subprocess
+import sys
 from ariba import common, card_record, vfdb_parser, megares_data_finder, megares_zip_parser
 
 
@@ -186,6 +188,19 @@ class RefGenesGetter:
         print('and in your methods say that version', self.version, 'of the database was used')
 
 
+    @classmethod
+    def _get_genetic_epi_database_from_bitbucket(cls, db_name, outdir, git_commit=None):
+        assert db_name in {'plasmidfinder', 'resfinder', 'virulencefinder'}
+        cmd = 'git clone ' + 'https://bitbucket.org/genomicepidemiology/' + db_name + '_db.git ' + outdir
+        common.syscall(cmd)
+
+        if git_commit is not None:
+            common.syscall('cd ' + outdir + ' && git checkout ' + git_commit)
+
+        print('Using this git commit for ' + db_name + ' database:')
+        subprocess.check_call('cd ' + outdir + ' && git log -n 1', shell=True)
+
+
     def _get_from_resfinder(self, outprefix):
         outprefix = os.path.abspath(outprefix)
         final_fasta = outprefix + '.fa'
@@ -193,17 +208,22 @@ class RefGenesGetter:
         tmpdir = outprefix + '.tmp.download'
         current_dir = os.getcwd()
 
-        try:
-            os.mkdir(tmpdir)
+        if self.version =='old':
+            try:
+                os.mkdir(tmpdir)
+                os.chdir(tmpdir)
+            except:
+                raise Error('Error mkdir/chdir ' + tmpdir)
+
+            zipfile = 'resfinder.zip'
+            cmd = 'curl -X POST --data "folder=resfinder&filename=resfinder.zip" -o ' + zipfile + ' https://cge.cbs.dtu.dk/cge/download_data.php'
+            print('Downloading data with:', cmd, sep='\n')
+            common.syscall(cmd)
+            common.syscall('unzip ' + zipfile)
+        else:
+            RefGenesGetter._get_genetic_epi_database_from_bitbucket('resfinder', tmpdir, git_commit=self.version)
             os.chdir(tmpdir)
-        except:
-            raise Error('Error mkdir/chdir ' + tmpdir)
 
-        zipfile = 'resfinder.zip'
-        cmd = 'curl -X POST --data "folder=resfinder&filename=resfinder.zip" -o ' + zipfile + ' https://cge.cbs.dtu.dk/cge/download_data.php'
-        print('Downloading data with:', cmd, sep='\n')
-        common.syscall(cmd)
-        common.syscall('unzip ' + zipfile)
 
         print('Combining downloaded fasta files...')
         fout_fa = pyfastaq.utils.open_file_write(final_fasta)
@@ -222,7 +242,7 @@ class RefGenesGetter:
                     except:
                         description = '.'
 
-                    # names are not unique across the files
+                    # names are not unique across the files
                     if seq.id in used_names:
                         used_names[seq.id] += 1
                         seq.id += '_' + str(used_names[seq.id])
@@ -310,17 +330,21 @@ class RefGenesGetter:
         tmpdir = outprefix + '.tmp.download'
         current_dir = os.getcwd()
 
-        try:
-            os.mkdir(tmpdir)
-            os.chdir(tmpdir)
-        except:
-            raise Error('Error mkdir/chdir ' + tmpdir)
+        if self.version == 'old':
+            try:
+                os.mkdir(tmpdir)
+                os.chdir(tmpdir)
+            except:
+                raise Error('Error mkdir/chdir ' + tmpdir)
 
-        zipfile = 'plasmidfinder.zip'
-        cmd = 'curl -X POST --data "folder=plasmidfinder&filename=plasmidfinder.zip" -o ' + zipfile + ' https://cge.cbs.dtu.dk/cge/download_data.php'
-        print('Downloading data with:', cmd, sep='\n')
-        common.syscall(cmd)
-        common.syscall('unzip ' + zipfile)
+            zipfile = 'plasmidfinder.zip'
+            cmd = 'curl -X POST --data "folder=plasmidfinder&filename=plasmidfinder.zip" -o ' + zipfile + ' https://cge.cbs.dtu.dk/cge/download_data.php'
+            print('Downloading data with:', cmd, sep='\n')
+            common.syscall(cmd)
+            common.syscall('unzip ' + zipfile)
+        else:
+            RefGenesGetter._get_genetic_epi_database_from_bitbucket('plasmidfinder', tmpdir, git_commit=self.version)
+            os.chdir(tmpdir)
 
         print('Combining downloaded fasta files...')
         fout_fa = pyfastaq.utils.open_file_write(final_fasta)
@@ -357,8 +381,13 @@ class RefGenesGetter:
 
 
     def _get_from_srst2_argannot(self, outprefix):
-        srst2_version = '0.2.0'
-        srst2_url = 'https://github.com/katholt/srst2/raw/v' + srst2_version + '/data/ARGannot.r1.fasta'
+        if self.version is None:
+            self.version = 'r2'
+        if self.version not in {'r1', 'r2'}:
+            raise Error('srst2_argannot version must be r1 or r2. Got this: ' + self.version)
+
+        version_string = '.r1' if self.version == 'r1' else '_r2'
+        srst2_url = 'https://raw.githubusercontent.com/katholt/srst2/master/data/ARGannot' + version_string + '.fasta'
         srst2_fa = outprefix + '.original.fa'
         command = 'wget -O ' + srst2_fa + ' ' + srst2_url
         common.syscall(command, verbose=True)
@@ -389,7 +418,9 @@ class RefGenesGetter:
         print('If you use this downloaded data, please cite:')
         print('"SRST2: Rapid genomic surveillance for public health and hospital microbiology labs",\nInouye et al 2014, Genome Medicine, PMID: 25422674\n')
         print(argannot_ref)
-        print('and in your methods say that the ARG-ANNOT sequences were used from version', srst2_version, 'of SRST2.')
+        # Use to also output the version of SRST2 here, but the r2 version of their
+        # fasta file was made after SRST2 release 0.2.0. At the time of writing this,
+        # 0.2.0 is the latest release, ie r2 isn't in an SRST2 release.
 
 
     def _get_from_vfdb_core(self, outprefix):
@@ -427,6 +458,31 @@ class RefGenesGetter:
         print('"VFDB 2016: hierarchical and refined dataset for big data analysis-10 years on",\nChen LH et al 2016, Nucleic Acids Res. 44(Database issue):D694-D697. PMID: 26578559\n')
 
 
+    @classmethod
+    def _fix_virulencefinder_fasta_file(cls, infile, outfile):
+        '''Some line breaks are missing in the FASTA files from
+        viruslence finder. Which means there are lines like this:
+        AAGATCCAATAACTGAAGATGTTGAACAAACAATTCATAATATTTATGGTCAATATGCTATTTTCGTTGA
+        AGGTGTTGCGCATTTACCTGGACATCTCTCTCCATTATTAAAAAAATTACTACTTAAATCTTTATAA>coa:1:BA000018.3
+        ATGAAAAAGCAAATAATTTCGCTAGGCGCATTAGCAGTTGCATCTAGCTTATTTACATGGGATAACAAAG
+        and therefore the sequences are messed up when we parse them. Also
+        one has a > at the end, then the seq name on the next line.
+        This function fixes the file by adding line breaks'''
+        with open(infile) as f_in, open(outfile, 'w') as f_out:
+            for line in f_in:
+                if line.startswith('>') or '>' not in line:
+                    print(line, end='', file=f_out)
+                elif line.endswith('>\n'):
+                    print('WARNING: found line with ">" at the end! Fixing. Line:' + line.rstrip() + ' in file ' + infile, file=sys.stderr)
+                    print(line.rstrip('>\n'), file=f_out)
+                    print('>', end='', file=f_out)
+                else:
+                    print('WARNING: found line with ">" not at the start! Fixing. Line:' + line.rstrip() + ' in file ' + infile, file=sys.stderr)
+                    line1, line2 = line.split('>')
+                    print(line1, file=f_out)
+                    print('>', line2, sep='', end='', file=f_out)
+
+
     def _get_from_virulencefinder(self, outprefix):
         outprefix = os.path.abspath(outprefix)
         final_fasta = outprefix + '.fa'
@@ -434,17 +490,21 @@ class RefGenesGetter:
         tmpdir = outprefix + '.tmp.download'
         current_dir = os.getcwd()
 
-        try:
-            os.mkdir(tmpdir)
-            os.chdir(tmpdir)
-        except:
-            raise Error('Error mkdir/chdir ' + tmpdir)
+        if self.version == 'old':
+            try:
+                os.mkdir(tmpdir)
+                os.chdir(tmpdir)
+            except:
+                raise Error('Error mkdir/chdir ' + tmpdir)
 
-        zipfile = 'plasmidfinder.zip'
-        cmd = 'curl -X POST --data "folder=virulencefinder&filename=virulencefinder.zip" -o ' + zipfile + ' https://cge.cbs.dtu.dk/cge/download_data.php'
-        print('Downloading data with:', cmd, sep='\n')
-        common.syscall(cmd)
-        common.syscall('unzip ' + zipfile)
+            zipfile = 'virulencefinder.zip'
+            cmd = 'curl -X POST --data "folder=virulencefinder&filename=virulencefinder.zip" -o ' + zipfile + ' https://cge.cbs.dtu.dk/cge/download_data.php'
+            print('Downloading data with:', cmd, sep='\n')
+            common.syscall(cmd)
+            common.syscall('unzip ' + zipfile)
+        else:
+            RefGenesGetter._get_genetic_epi_database_from_bitbucket('virulencefinder', tmpdir, git_commit=self.version)
+            os.chdir(tmpdir)
 
         print('Combining downloaded fasta files...')
         fout_fa = pyfastaq.utils.open_file_write(final_fasta)
@@ -454,7 +514,9 @@ class RefGenesGetter:
         for filename in os.listdir(tmpdir):
             if filename.endswith('.fsa'):
                 print('   ', filename)
-                file_reader = pyfastaq.sequences.file_reader(os.path.join(tmpdir, filename))
+                fix_file = os.path.join(tmpdir, filename + '.fix.fsa')
+                RefGenesGetter._fix_virulencefinder_fasta_file(os.path.join(tmpdir, filename), fix_file)
+                file_reader = pyfastaq.sequences.file_reader(fix_file)
                 for seq in file_reader:
                     original_id = seq.id
                     seq.id = seq.id.replace('_', '.', 1)
